import 'package:osam/osam.dart';
import 'package:osam_flutter/src/navigation/osam_route.dart';

import '../../osam_flutter.dart';

part 'navigation.g.dart';

class NoRouteException implements Exception {}

@HiveType(typeId: 222)
class Navigation {
  @HiveField(0)
  final List<OsamRoute<Object>> _routes;
  final PropertyNotifier<List<OsamRoute<Object>>> routes;

  Navigation(this._routes) : routes = PropertyNotifier(_routes);

  Future<R> push<R>(OsamRoute<R> route) {
    routes.update((value) => value.add(route));
    return route.whenComplete;
  }

  void pop() => routes.update((value) => value.removeLast());

  T findRoute<T extends OsamRoute>() =>
      _routes.firstWhere((element) => element is T);

  Future<R> pushReplacement<R>(OsamRoute<R> route) {
    routes.update((value) {
      value.removeLast();
      value.add(route);
    });
    return route.whenComplete;
  }

  Future<R> pushAndRemoveUntil<R>(OsamRoute<R> route) {
    routes.update((value) {
      value.clear();
      value.add(route);
    });
    return route.whenComplete;
  }

  void popUntil<T extends OsamRoute>() => routes.update((value) {
        if (value.contains(T)) {
          while (!(value.last is T)) {
            value.removeLast();
          }
        }
      });
}

@HiveType(typeId: 221)
class TabNavigation {
  @HiveField(0)
  String _tab;
  final PropertyNotifier<String> tabNotifier;

  TabNavigation(this._tab) : tabNotifier = PropertyNotifier(_tab);

  set tab(String tab) {
    _tab = tab;
    tabNotifier.value = _tab;
  }
}
